@@ -23,6 +23,14 @@
 #  endif
 # endif
 
+#else
+
+#if 0
+# if defined(__FreeBSD__)
+#  define BROKEN_RTLD_NEXT
+# endif
+#endif
+
 #endif
 
 #define GLIBC_MINOR __GLIBC_MINOR__
@@ -40,10 +48,23 @@
 # error PATH_MAX not defined!
 #endif
 
-#ifndef OUTSIDE_LIBSANDBOX
-# define SB_STATIC static
-#else
-# define SB_STATIC
+#if !HAVE_DLVSYM
+# define dlvsym(_lib, _sym, _ver) dlsym(_lib, _sym)
 #endif
 
+#if HAVE_DLVSYM
+/* Taken from glibc */
+# define symbol_version(_real, _name, _version) \
+	__asm__ (".symver " #_real "," #_name "@" #_version)
+# define default_symbol_version(_real, _name, _version) \
+	__asm__ (".symver " #_real "," #_name "@@" #_version)
+#endif
+
+/* Taken from glibc */
+# define strong_alias(_name, _aliasname) \
+	extern __typeof (_name) _aliasname __attribute__ ((alias (#_name)));
+# define weak_alias(_name, _aliasname) \
+	extern __typeof (_name) _aliasname __attribute__ ((weak, alias (#_name)));
+
+
 #endif
