@@ -29,22 +29,17 @@
  *
  */
 
-/* Uncomment below to enable the use of strtok_r(). */
-#define REENTRANT_STRTOK 1
-
 /* Uncomment below to enable memory debugging. */
 /* #define SB_MEM_DEBUG 1 */
 
 #define open   xxx_open
 #define open64 xxx_open64
 
-#if !defined(BROKEN_RTLD_NEXT) && defined(HAVE_RTLD_NEXT)
-# define USE_RTLD_NEXT
-#endif
+#include "config.h"
 
 /* Better way would be to only define _GNU_SOURCE when __GLIBC__ is defined,
  * but including features.h and then defining _GNU_SOURCE do not work */
-#if defined(USE_RTLD_NEXT)
+#if defined(HAVE_RTLD_NEXT)
 # define _GNU_SOURCE
 #endif
 #include <dirent.h>
@@ -63,9 +58,12 @@
 #include <unistd.h>
 #include <utime.h>
 
-#include "config.h"
 #include "localdecls.h"
 
+#if !defined(BROKEN_RTLD_NEXT) && defined(HAVE_RTLD_NEXT)
+# define USE_RTLD_NEXT
+#endif
+
 #ifdef SB_MEM_DEBUG
 # include <mcheck.h>
 #endif
@@ -904,6 +902,7 @@ char *egetcwd(char *buf, size_t size)
 {
 	struct stat st;
 	char *tmpbuf;
+	int old_errno;
 
 	/* Need to disable sandbox, as on non-linux libc's, opendir() is
 	 * used by some getcwd() implementations and resolves to the sandbox
@@ -914,17 +913,37 @@ char *egetcwd(char *buf, size_t size)
 	errno = 0;
 	tmpbuf = true_getcwd_DEFAULT(buf, size);
 	sandbox_on = 1;
-	if (tmpbuf)
+	if (tmpbuf) {
+		old_errno = errno;
 		lstat(buf, &st);
 
-	if ((tmpbuf) && (errno == ENOENT)) {
-		/* If lstat() failed with eerror = ENOENT, then its
-		 * possible that we are running on an older kernel
-		 * which had issues with returning invalid paths if
-		 * they got too long.
-		 */
-		free(tmpbuf);
-			
+		if (errno == ENOENT) {
+			/* If lstat() failed with eerror = ENOENT, then its
+			 * possible that we are running on an older kernel
+			 * which had issues with returning invalid paths if
+			 * they got too long.
+			 */
+		  	errno = ENAMETOOLONG;
+			free(tmpbuf);
+			return NULL;
+		} else if (errno != 0) {
+			/* Not sure if we should quit here, but I guess if
+			 * lstat() fails, getcwd could have messed up.
+			 */
+			free(tmpbuf);
+			return NULL;
+		}
+
+		errno = old_errno;
+	}
+
+	/* Make sure we do not return garbage if the current libc or kernel's
+	 * getcwd() is buggy.
+	 */
+	if (errno != 0) {
+	  	if (tmpbuf)
+			free(tmpbuf);
+
 		return NULL;
 	}
 
@@ -1013,7 +1032,7 @@ static void init_env_entries(char ***prefixes_array, int *prefixes_num, const ch
 		goto error;
 	buffer_ptr = buffer;
 
-#ifdef REENTRANT_STRTOK
+#ifdef HAVE_STRTOK_R
 	token = strtok_r(buffer_ptr, ":", &buffer_ptr);
 #else
 	token = strtok(buffer_ptr, ":");
@@ -1041,7 +1060,7 @@ static void init_env_entries(char ***prefixes_array, int *prefixes_num, const ch
 			}
 		}
 
-#ifdef REENTRANT_STRTOK
+#ifdef HAVE_STRTOK_R
 		token = strtok_r(NULL, ":", &buffer_ptr);
 #else
 		token = strtok(NULL, ":");
